#include "cell.h"
#include "gridentity.h"
#include "grid.h"
#include "helper.h"
#include <stdio.h>
#include <qmessagebox.h>
#include <qdom.h> 
#include <qfile.h>
#include <qstring.h>
#include <qtextstream.h> 

// Constructor
Grid::Grid(QString strFilename)
{
	Initialise(strFilename);
	Populate(strFilename);
}

// Destructor
Grid::~Grid()
{
	delete[] m_Squares;
	delete[] m_Rows;
	delete[] m_Columns;
	delete[] m_Cells;
}

void Grid::Initialise(QString strFilename)
{
	// Open the xml file
	QDomDocument xmlDoc;
	QFile file(strFilename);
    if(!file.open(IO_ReadOnly))
	{
		QMessageBox::critical(0, "Sudoku Solver", "Couldn't open file '" + strFilename + "'.");
        return;
	}
    if (!xmlDoc.setContent(&file))
	{
        file.close();
		QMessageBox::critical(0, "Sudoku Solver", "Couldn't open file '" + strFilename + "'.");
        return;
    }
    file.close();

	// Get the grid size
	QDomAttr xmlGridSize = xmlDoc.documentElement().attributeNode("size");
	m_intGridSize = atoi(xmlGridSize.value());

	QDomAttr xmlGridName = xmlDoc.documentElement().attributeNode("name");
	m_strGridName = xmlGridName.value();

	Reset();
}

void Grid::Reset()
{
	m_Squares = new GridEntity[m_intGridSize];
	m_Rows = new GridEntity[m_intGridSize];
	m_Columns = new GridEntity[m_intGridSize];
	for(int i=0; i<m_intGridSize; i++)
	{
		m_Squares[i].Initialise(m_intGridSize);
		m_Rows[i].Initialise(m_intGridSize);
		m_Columns[i].Initialise(m_intGridSize);
	}

	m_Cells = new Cell[m_intGridSize * m_intGridSize];
	for(int i=0; i<(m_intGridSize * m_intGridSize); i++)
		m_Cells[i].Initialise(m_intGridSize);

	m_DifficultyLevel = NOTSET;
	m_blnSolved = false;
}

int Grid::GetGridSize()
{
	return m_intGridSize;
}

int Grid::GetDifficultyLevel()
{
	return m_DifficultyLevel;
}


void Grid::SetDifficultyLevel(int intLevel)
{
	// Can only increase difficulty level
	if(m_DifficultyLevel < intLevel)
		m_DifficultyLevel = intLevel;
}

QString Grid::GetGridName()
{
	return m_strGridName;
}

void Grid::SetGridName(QString strGridName)
{
	m_strGridName = strGridName;
}

bool Grid::IsSolved()
{
	return m_blnSolved;
}

void Grid::Print()
{
	for(int i=0; i<m_intGridSize; i++)
	{
		for(int j=0; j<m_intGridSize; j++)
			printf("%i ", m_Cells[(i*m_intGridSize)+j].GetValue());

		printf("\n");
	}
	printf("\n");

	if(IsSolved())
	{
		printf("The difficulty level of the solution was: ");
		switch(m_DifficultyLevel)
		{
			case EASY:
				printf("easy");
				break;
			case AVERAGE:
				printf("average");
				break;
			case DIFFICULT:
				printf("difficult");
				break;
			default:
				printf("undefined");
		}
		printf("\n");
	}

}

void Grid::Output(QString strFilename)
{
	const int intIndent = 4;

	QDomDocument xmlDoc;

	// Processing Instruction
	QDomNode xmlNode = xmlDoc.createProcessingInstruction("xml", "version=\"1.0\" encoding=\"iso-8859-1\"");
	xmlDoc.appendChild(xmlNode);

	// Grid element
	QDomElement xmlGrid = xmlDoc.createElement("grid");
	QDomAttr xmlAttr = xmlDoc.createAttribute("size");
	QString strGridSize;
	strGridSize.setNum(m_intGridSize);
	xmlAttr.setValue(strGridSize);
	xmlGrid.setAttributeNode(xmlAttr);
	xmlAttr = xmlDoc.createAttribute("name");
	xmlAttr.setValue(m_strGridName);
	xmlGrid.setAttributeNode(xmlAttr);
	xmlDoc.appendChild(xmlGrid);

	// Cell elements
	for(int i=0; i<(m_intGridSize*m_intGridSize); i++)
	{
		QDomElement xmlCell = xmlDoc.createElement("cell");
		QDomAttr xmlAttr = xmlDoc.createAttribute("index");
		QString strTemp;
		strTemp.setNum(i);
		xmlAttr.setValue(strTemp);
		xmlCell.setAttributeNode(xmlAttr);
		if(m_Cells[i].GetValue() > 0)
		{
			strTemp.setNum(m_Cells[i].GetValue());
			QDomText xmlText = xmlDoc.createTextNode(strTemp);
			xmlCell.appendChild(xmlText);
			xmlGrid.appendChild(xmlCell);
		}
	}

	QFile fGridXml(strFilename);
	if(!fGridXml.open(IO_WriteOnly))
	{
		QMessageBox::critical(0, "Sudoku Solver", "Couldn't open file '" + strFilename + "'.");
        return;
	}

	QTextStream out(&fGridXml);
	xmlDoc.save(out, intIndent);

}

void Grid::Solve()
{
	const int CELL_COUNT = (m_intGridSize * m_intGridSize);
	const QString TEMP_GRID_FILE = "/tmp/newgridtemp.xml";

	// ====================================================================
	// Work through each cell, restarting when each new value is found,
	// until all values have been found.
	// ====================================================================
	int i = 0;
	do
	{
		// If the value of this cell has not already been established...
		if(m_Cells[i].GetValue() == 0)
		{
			// Calculate the possible values for the cell (returning the value if found)
			if(CalculateCellPossibleValues(i) > 0)
				i=-1; // New cell value found so restart solution process
		}

		i++;

	} while(i < CELL_COUNT);

	// Check if the puzzle is solved and if so exit.
	if(CheckIfSolved() == true)
	{
		SetDifficultyLevel(EASY);

		Output(TEMP_GRID_FILE);
		return;
	}

	// ====================================================================
	// Look for cell values using more subtle methods
	// ====================================================================

	// Naked Pairs
	if(FindNakedPair())
	{
		SetDifficultyLevel(DIFFICULT);

		// New values were found
		// Check if the puzzle is solved and if so exit.
		if(CheckIfSolved() == true)
		{
			Output(TEMP_GRID_FILE);
			return;
		}

		// Puzzle still not solved so restart by recursively calling Solve
		Solve();
	}
}

int Grid::CalculateCellPossibleValues(int intCurrentCellIndex)
{
	int intReturnCellValue = 0;

	// ====================================================================
	// Sole Candidate
	// Attempt to find any cells where only one possible value is available
	// based upon the current row, column and square.
	// ====================================================================
	intReturnCellValue = FindSingleCandidate(intCurrentCellIndex);
	if(intReturnCellValue > 0)
	{
		// printf("Cell No %i -> VALUE FOUND = %i\n", intCurrentCellIndex, intReturnCellValue);
		return intReturnCellValue;
	}

	// ====================================================================
	// Hidden Single
	// Cell can only have a single value based upon cells in nearby rows,
	// columns and squares.
	// ====================================================================
	intReturnCellValue = FindHiddenSingle(intCurrentCellIndex);
	if(intReturnCellValue > 0)
	{
		printf("Found hidden single...\n");
		SetDifficultyLevel(AVERAGE);
		return intReturnCellValue;
	}

	// ====================================================================
	// Unfound.
	// Cell value not found so return 0.
	// ====================================================================
	return 0;

}

int Grid::FindSingleCandidate(int intCurrentCellIndex)
{

	// Get values of all cells in the current row and update current cell 'possibles'
	int intRowNo = GetRowNumber(intCurrentCellIndex, m_intGridSize);
	for(int i=0; i<m_intGridSize; i++)
	{
		int intTempCellValue = m_Cells[m_Rows[intRowNo].GetCellIndex(i)].GetValue();
		if(intTempCellValue != 0)
			m_Cells[intCurrentCellIndex].SetPossibleValue(intTempCellValue, false);
	}
		
	// If the cell value has been found, exit and return it
	int intCurrentCellValue = m_Cells[intCurrentCellIndex].GetValue();
	if(intCurrentCellValue > 0)
		return intCurrentCellValue;

	// Get values of all cells in the current column and update current cell 'possibles'
	int intColNo = GetColumnNumber(intCurrentCellIndex, m_intGridSize);
	for(int i=0; i<m_intGridSize; i++)
	{
		int intTempCellValue = m_Cells[m_Columns[intColNo].GetCellIndex(i)].GetValue();
		if(intTempCellValue != 0)
			m_Cells[intCurrentCellIndex].SetPossibleValue(intTempCellValue, false);
	}

	// If the cell value has been found, exit and return it
	intCurrentCellValue = m_Cells[intCurrentCellIndex].GetValue();
	if(intCurrentCellValue > 0)
		return intCurrentCellValue;

	// Get values of all cells in the current square
	int intSquareIndex = m_Cells[intCurrentCellIndex].GetSquareIndex();
	for(int i=0; i<m_intGridSize; i++)
	{
		int intTempCellValue = m_Cells[m_Squares[intSquareIndex].GetCellIndex(i)].GetValue();
		if(intTempCellValue != 0)
			m_Cells[intCurrentCellIndex].SetPossibleValue(intTempCellValue, false);
	}

	// If the cell value has been found, exit and return it
	intCurrentCellValue = m_Cells[intCurrentCellIndex].GetValue();
	if(intCurrentCellValue > 0)
		return intCurrentCellValue;


	// Cell value not found so return 0
	return 0;

}

int Grid::FindHiddenSingle(int intCurrentCellIndex)
{
	bool blnCheckColumn = true;
	bool blnCheckRow = true;
	int intReturnValue = 0;

	// Get the current possible cell values
	bool* blnPossibles = m_Cells[intCurrentCellIndex].GetPossibleValues();

	int intRowNo = GetRowNumber(intCurrentCellIndex, m_intGridSize);
	int intColNo = GetColumnNumber(intCurrentCellIndex, m_intGridSize);
	int intCurrentSquare = GetSquareNumber(intCurrentCellIndex, m_intGridSize);

	// Check if this column/square is suitable for a hidden single check
	for(int i = 0; i<m_intGridSize; i++)
	{
		int intColCellIndex = m_Columns[intColNo].GetCellIndex(i);
		if(intColCellIndex != intCurrentCellIndex && m_Cells[intColCellIndex].GetSquareIndex() == intCurrentSquare)
			if(m_Cells[intColCellIndex].GetValue() == 0)
			{
				// Another cell in the current column/square is empty so cannot
				// attempt a hidden single match on this column/square.
				blnCheckColumn = false;
			}
	}

	// Check if this row/square is suitable for a hidden single check
	for(int i = 0; i<m_intGridSize; i++)
	{
		int intRowCellIndex = m_Rows[intRowNo].GetCellIndex(i);
		if(intRowCellIndex != intCurrentCellIndex && m_Cells[intRowCellIndex].GetSquareIndex() == intCurrentSquare)
			if(m_Cells[intRowCellIndex].GetValue() == 0)
			{
				// Another cell in the current row/square is empty so cannot
				// attempt a hidden single match on this row/square.
				blnCheckRow = false;
			}
	}
	
	// If the current cell cannot be checked for a hidden single based on
	// row or column the exit now
	if(blnCheckRow == true || blnCheckColumn == true)
	{
		// printf("Calling FindHiddenSingle\n");
		// Print();
		for(int i=0; i<m_intGridSize; i++)
			if(blnPossibles[i] == true)
			{
				// For each possible value, check for hidden singles
				bool blnFound = false;
				int intPossibleValueForCurrentCell = i+1;
	
				// If all other cells in the current column/square are entered and
				// a possible value is present in both other columns in the square
				// a hidden single is found
				if(blnCheckColumn)
				{
					int* intColsInSquare = GetColumnsInSquare(intCurrentCellIndex, m_intGridSize, true);
					for(int n=0; n<sqrt(m_intGridSize)-1; n++)
					{
						for(int j=0; j<m_intGridSize; j++)
						{
							int intTempCellIndex = m_Columns[intColsInSquare[n]].GetCellIndex(j);
							if((m_Cells[intTempCellIndex].GetSquareIndex() == intCurrentSquare) &&
												 (m_Cells[intTempCellIndex].GetValue() == intPossibleValueForCurrentCell))
								blnFound = true;
						}

						if(blnFound == false)
							break;
					}

					if(blnFound == true)
						intReturnValue = intPossibleValueForCurrentCell;

					delete intColsInSquare;
				}
			
				// If all other cells in the current row/square are entered and
				// a possible value is present in both other rows in the square
				// a hidden single is found
				if(blnCheckRow && intReturnValue == 0)
				{
					int* intRowsInSquare = GetRowsInSquare(intColNo, m_intGridSize, true);

					for(int n=0; n<sqrt(m_intGridSize)-1; n++)
					{
						for(int j=0; j<m_intGridSize; j++)
						{
							int intTempCellIndex = m_Rows[intRowsInSquare[n]].GetCellIndex(j);
							if(m_Cells[intTempCellIndex].GetSquareIndex() == intCurrentSquare &&
												m_Cells[intTempCellIndex].GetValue() == intPossibleValueForCurrentCell)
								blnFound = true;
						}

						if(blnFound == false)
							break;
					}

					if(blnFound == true)
						intReturnValue = intPossibleValueForCurrentCell;

					delete intRowsInSquare;
				}
			}
	}

	return intReturnValue;
}

bool Grid::FindNakedPair()
{
	// =================================================================
	// If two cells in the same row/column/square have only the same two
	// possible values, those values can be removed from the list of
	// possible values in all other cells in the same row/column/square
	// =================================================================
	bool blnReturnValue = false;
	int intFirstPossible = 0;
	int intSecondPossible = 0;

	// Check each row
	for(int intRowNo = 0; intRowNo<m_intGridSize; intRowNo++)
	{
		// Check each cell in that row
		for(int intColNo = 0; intColNo<m_intGridSize; intColNo++)
		{
			intFirstPossible = -1;
			intSecondPossible = -1;
			int intCellIndex = (intRowNo * m_intGridSize) + intColNo;

			if(m_Cells[intCellIndex].GetValue() == 0 && m_Cells[intCellIndex].GetNumberOfPossibles() == 2)
			{
				// Cell has not been solved yet and has 2 possible values
				bool* blnPossibles = m_Cells[intCellIndex].GetPossibleValues();
				for(int intPossibleIndex=0; intPossibleIndex<m_intGridSize; intPossibleIndex++)
				{
					if(blnPossibles[intPossibleIndex] == true)
						if(intFirstPossible == -1)
							intFirstPossible = intPossibleIndex + 1;
						else
						{
							intSecondPossible = intPossibleIndex + 1;
							break;
						}
				}

				if(FindNakedPair_Row(intCellIndex, intColNo, intRowNo, intFirstPossible, intSecondPossible))
					return true;

				if(FindNakedPair_Column(intCellIndex, intColNo, intRowNo, intFirstPossible, intSecondPossible))
					return true;

				//if(FindNakedPair_Square(intCellIndex, intColNo, intRowNo, intFirstPossible, intSecondPossible))
				//	return true;
			}
		}
	}

	return blnReturnValue;

}

bool Grid::FindNakedPair_Row(int intCellIndex, int intColNo, int intRowNo, int intFirstPossible, int intSecondPossible)
{
	int intFirstCell = -1;
	int intSecondCell = -1;
	bool blnFound = false;

	if(intColNo < m_intGridSize - 1)
	{
		// Check the remaining cells in the row for the same possible values
		for(int intColNo2=intColNo+1; intColNo2<m_intGridSize; intColNo2++)
		{
			int intCurrentCellNo = (intRowNo * m_intGridSize) + intColNo2;
			if(m_Cells[intCurrentCellNo].GetNumberOfPossibles() == 2)
			{
				bool* blnPossibles = m_Cells[intCurrentCellNo].GetPossibleValues();
				if(blnPossibles[intFirstPossible-1] == true && blnPossibles[intSecondPossible-1] == true)
				{
					// Naked Pair found.
					intFirstCell = intCellIndex;
					intSecondCell = (intRowNo * m_intGridSize) + intColNo2;
					blnFound = true;
					break;
				}
			}
		}
	}
	
	if(blnFound)
	{
		// Naked Pair found, so the two values can be removed as possibles
		// from all other cells in the row.
		blnFound = false;
		for(int intColNo2=0; intColNo2<m_intGridSize; intColNo2++)
		{
			int intCurrentCellNo = (intRowNo * m_intGridSize) + intColNo2;
			if(intCurrentCellNo != intFirstCell && intCurrentCellNo != intSecondCell 
														&& m_Cells[intCurrentCellNo].GetValue() == 0)
			{
				m_Cells[intCurrentCellNo].SetPossibleValue(intFirstPossible, false);
				m_Cells[intCurrentCellNo].SetPossibleValue(intSecondPossible, false);
				if(m_Cells[intCurrentCellNo].GetValue() > 0)
				{
					// A new value has been found
					return true;
				}
			}
		}
	}

	return false;
}

bool Grid::FindNakedPair_Column(int intCellIndex, int intColNo, int intRowNo, int intFirstPossible, int intSecondPossible)
{
	int intFirstCell = -1;
	int intSecondCell = -1;
	bool blnFound = false;

	if(intRowNo < m_intGridSize - 1)
	{
		// Check the remaining cells in the column for the same possible values
		for(int intRowNo2=intRowNo+1; intRowNo2<m_intGridSize; intRowNo2++)
		{
			int intCurrentCellNo = (intRowNo2 * m_intGridSize) + intColNo;
			if(m_Cells[intCurrentCellNo].GetNumberOfPossibles() == 2)
			{
				bool* blnPossibles = m_Cells[intCurrentCellNo].GetPossibleValues();
				if(blnPossibles[intFirstPossible-1] == true && blnPossibles[intSecondPossible-1] == true)
				{
					// Naked Pair found.
					intFirstCell = intCellIndex;
					intSecondCell = (intRowNo2 * m_intGridSize) + intColNo;
					blnFound = true;
					break;
				}
			}
		}
	}
	
	if(blnFound)
	{
		// Naked Pair found, so the two values can be removed as possibles
		// from all other cells in the row.
		blnFound = false;
		for(int intRowNo2=0; intRowNo2<m_intGridSize; intRowNo2++)
		{
			int intCurrentCellNo = (intRowNo2 * m_intGridSize) + intColNo;
			if(intCurrentCellNo != intFirstCell && intCurrentCellNo != intSecondCell 
														&& m_Cells[intCurrentCellNo].GetValue() == 0)
			{
				m_Cells[intCurrentCellNo].SetPossibleValue(intFirstPossible, false);
				m_Cells[intCurrentCellNo].SetPossibleValue(intSecondPossible, false);
				if(m_Cells[intCurrentCellNo].GetValue() > 0)
				{
					// A new value has been found
					return true;
				}
			}
		}
	}

	return false;
}

/*
bool Grid::FindNakedPair_Square(int intCellIndex, int intColNo, int intRowNo, int intFirstPossible, int intSecondPossible)
{
	// Not implemented
	return false;
}
*/

bool Grid::CheckIfSolved()
{
	for(int i=0; i<(m_intGridSize*m_intGridSize); i++)
		if(m_Cells[i].GetValue() == 0)
			return false;

	// All cells are populated, but is the 'solution' valid?
	if(ValidateSolution())
		m_blnSolved = true;

	return m_blnSolved;
}

bool Grid::ValidateSolution()
{
	// Test each cell to ensure that no other cell 
	// in the same row/column/square has the same value
	for(int i=0; i<(m_intGridSize*m_intGridSize); i++)
	{
		// Find the value of the current cell
		int intCellValue = m_Cells[i].GetValue();

		// Find the row, column and square number of the current cell
		int intRowNo = GetRowNumber(i, m_intGridSize);
		int intColumnNo = GetColumnNumber(i, m_intGridSize);
		int intSquareNo = GetSquareNumber(i, m_intGridSize);
		
		// Check each 'other' cell in the row, column, square
		for(int intEntityIndex=0; intEntityIndex<m_intGridSize; intEntityIndex++)
		{
			// Rows
			int intCellIndex = m_Rows[intRowNo].GetCellIndex(intEntityIndex);
			if(intCellIndex != i && m_Cells[intCellIndex].GetValue() == intCellValue)
				return false;	// Invalid solution

			// Columns
			intCellIndex = m_Columns[intColumnNo].GetCellIndex(intEntityIndex);
			if(intCellIndex != i && m_Cells[intCellIndex].GetValue() == intCellValue)
				return false;	// Invalid solution

			// Squares
			intCellIndex = m_Squares[intSquareNo].GetCellIndex(intEntityIndex);
			if(intCellIndex != i && m_Cells[intCellIndex].GetValue() == intCellValue)
				return false;	// Invalid solution
		}
	}
	
	// Valid solution
	return true;
}

bool Grid::Populate(QString strFilename)
{
	// Open the xml file
	QDomDocument xmlDoc;
	QFile file(strFilename);
    if(!file.open(IO_ReadOnly))
        return false;
    if (!xmlDoc.setContent(&file))
	{
        file.close();
		QMessageBox::critical(0, "Sudoku Solver", "Couldn't open file '" + strFilename + "'.");
        return false;
    }
    file.close();

	// Read the contents
	QDomNodeList xmlCellList = xmlDoc.elementsByTagName("cell");
	int intCellCount = int(xmlCellList.length());
	if(intCellCount > 0)
	{
		for(int i=0; i<intCellCount; i++)
		{
			QDomElement xmlCell = xmlCellList.item(i).toElement();
			QDomAttr xmlIndex = xmlCell.attributeNode("index");
			m_Cells[atoi(xmlIndex.value())].SetValue(atoi(xmlCell.text()));
		}
	}

	InitialiseCells();

	return true;
}

bool Grid::Populate(int* GridArray)
{
	for(int i=0; i<m_intGridSize*m_intGridSize; i++)
	{
		m_Cells[i].SetValue(GridArray[i]);
	}

	InitialiseCells();

	return true;
}

void Grid::InitialiseCells()
{
	for(int i=0; i<(m_intGridSize*m_intGridSize); i++)
	{
		// Populate Squares
		int intSquareNo = GetSquareNumber(i, m_intGridSize);
		m_Cells[i].SetSquareIndex(intSquareNo);
		m_Squares[intSquareNo].AddCell(i);
		m_Squares[intSquareNo].SetIndex(intSquareNo);

		// Populate Rows
		int intRowNo = GetRowNumber(i, m_intGridSize);
		m_Cells[i].SetRowIndex(intRowNo);
		m_Rows[intRowNo].AddCell(i);
		m_Rows[intRowNo].SetIndex(intRowNo);

		// Populate Columns
		int intColNo = GetColumnNumber(i, m_intGridSize);
		m_Cells[i].SetColumnIndex(intColNo);
		m_Columns[intColNo].AddCell(i);
		m_Columns[intColNo].SetIndex(intColNo);

	}
}

void Grid::RotateLeft()
{
	int* intGridArray = new int[m_intGridSize * m_intGridSize];
	for(int intRow=0; intRow<m_intGridSize; intRow++)
	{
		for(int intCol=0; intCol<m_intGridSize; intCol++)
		{
			int intCurrentCell = GetCellIndex(intRow, intCol, m_intGridSize);
			int intNewCell = GetCellIndex(m_intGridSize - 1 - intCol, intRow, m_intGridSize);
			intGridArray[intNewCell] = m_Cells[intCurrentCell].GetValue();
		}
	}

	Reset();
	Populate(intGridArray);
	delete intGridArray;
}			

void Grid::RotateRight()
{
	int* intGridArray = new int[m_intGridSize * m_intGridSize];
	for(int intRow=0; intRow<m_intGridSize; intRow++)
	{
		for(int intCol=0; intCol<m_intGridSize; intCol++)
		{
			int intCurrentCell = GetCellIndex(intRow, intCol, m_intGridSize);
			int intNewCell = GetCellIndex(intCol, m_intGridSize - 1 - intRow, m_intGridSize);
			intGridArray[intNewCell] = m_Cells[intCurrentCell].GetValue();
		}
	}

	Reset();
	Populate(intGridArray);
	delete intGridArray;
}

void Grid::FlipHorizontally()
{
	int* intGridArray = new int[m_intGridSize * m_intGridSize];
	int intMidRow = abs(m_intGridSize / 2);
	for(int i=0; i<intMidRow; i++)
	{
		int intFirstRow = i;
		int intSecondRow = m_intGridSize - i - 1;
		for(int intCol=0; intCol<m_intGridSize; intCol++)
		{
			int intCurrentCell = GetCellIndex(intFirstRow, intCol, m_intGridSize);
			int intNewCell = GetCellIndex(intSecondRow, intCol, m_intGridSize);
			intGridArray[intNewCell] = m_Cells[intCurrentCell].GetValue();
			intGridArray[intCurrentCell] = m_Cells[intNewCell].GetValue();
		}
	}

	for(int intCol=0; intCol<m_intGridSize; intCol++)
	{
		int intMidRowCell = GetCellIndex(intMidRow, intCol, m_intGridSize);
		intGridArray[intMidRowCell] = m_Cells[intMidRowCell].GetValue();
	}

	Reset();
	Populate(intGridArray);
	delete intGridArray;
}

void Grid::FlipVertically()
{
	int* intGridArray = new int[m_intGridSize * m_intGridSize];
	int intMidCol = abs(m_intGridSize / 2);
	for(int intCol=0; intCol<intMidCol; intCol++)
	{
		int intFirstCol = intCol;
		int intSecondCol = m_intGridSize - intCol - 1;
		for(int intRow=0; intRow<m_intGridSize; intRow++)
		{
			int intCurrentCell = GetCellIndex(intRow, intFirstCol, m_intGridSize);
			int intNewCell = GetCellIndex(intRow, intSecondCol, m_intGridSize);
			intGridArray[intNewCell] = m_Cells[intCurrentCell].GetValue();
			intGridArray[intCurrentCell] = m_Cells[intNewCell].GetValue();
		}
	}

	for(int intRow=0; intRow<m_intGridSize; intRow++)
	{
		int intMidColCell = GetCellIndex(intRow, intMidCol, m_intGridSize);
		intGridArray[intMidColCell] = m_Cells[intMidColCell].GetValue();
	}

	Reset();
	Populate(intGridArray);
	delete intGridArray;
}

void Grid::SwapNumbers(int intFirstNumber, int intSecondNumber)
{
	for(int i=0; i<(m_intGridSize*m_intGridSize); i++)
	{
		if(m_Cells[i].GetValue() == intFirstNumber)
			m_Cells[i].SetValue(intSecondNumber);
		else if(m_Cells[i].GetValue() == intSecondNumber)
			m_Cells[i].SetValue(intFirstNumber);
	}
}
